/* 08 Nov 1998*/
/*
 * cdev.c
 *
 * 08 Nov 1998	Rajesh Vaidheeswarran
 *
 * Copyright (c) 1998 Rajesh Vaidheeswarran
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Rajesh Vaidheeswarran.
 * 4. The name Rajesh Vaidheeswarran may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY RAJESH VAIDHEESWARRAN ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE RAJESH VAIDHEESWARRAN BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
  * Copyright (c) 1993 Terrence R. Lambert.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Terrence R. Lambert.
 * 4. The name Terrence R. Lambert may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TERRENCE R. LAMBERT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE TERRENCE R. LAMBERT BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/*
 * Fujitsu C-4110 application panel driver
 * See http://www.stack.nl/~wvengen/fujitsu/ and README for details
 *
 * Copyright by Willem van Engen <wvengen@stack.nl>
 * See COPYRIGHT for copyright
 * See CREDITS for credits
 *   
 */

#include <sys/types.h>
/*#include <sys/ioccom.h>*/
#include <sys/uio.h>
#include <sys/param.h>
#include <stdio.h>
#include <machine/cpufunc.h>
#include "cdev.h"

/*
 * This is the actual code for the system call... it can't be static because
 * it is exported to another part of the module... the only place it needs
 * to be referenced is the sysent we are interested in.
 *
 * To write your own system call using this as a template, you could strip
 * out this code and use the rest as a prototype module, changing only the
 * function names and the number of arguments to the call in the module
 * specific "sysent".
 *
 * You would have to use the "-R" option of "ld" to ensure a linkable file
 * if you were to do this, since you would need to combine multiple ".o"
 * files into a single ".o" file for use by "modload".
 */

int
mydev_open(dev_t dev, int flag, int otyp, struct proc *procp)
{
#ifdef DEBUG
	printf("apanel: openend\n");
#endif
    return (0);
}

int
mydev_close(dev_t dev, int flag, int otyp, struct proc *procp)
{
#ifdef DEBUG
	printf("apanel: closed\n");
#endif
    return (0);
}

int
mydev_write(dev_t dev,struct uio *uio,int ioflag)
{
  int rv;
  char part=0;
  unsigned int num=0;
  char buf[WRITEBUFSIZE];

  /* Check size to prevent buffer overflow */
  if ( uio->uio_resid > WRITEBUFSIZE) {
#ifdef DEBUG
    printf("apanel: uio->uio_resid(%d) > WRITEBUFSIZE(%d)\n",
                             uio->uio_resid,WRITEBUFSIZE);
#endif
    return 1;
  }
  /* Read into array buf */
  if ((rv=uiomove((caddr_t)buf,uio->uio_resid,uio))!=0) {
    return rv;
  }

  if (sscanf(buf,"%c%x",&part,&num)!=2) {
#ifdef DEBUG
    printf("apanel: incorrect format: '%s'\n",buf);
#endif
    return 1;
  }

  switch(part) {
  case 'L':
  case 'l':		/* led */
#ifdef DEBUG
    printf("apanel: setled %x\n",num);
#endif
    setled(num);
    return 0;
      
  case 'D':
  case 'd':			/* lcd display */
#ifdef DEBUG
    printf("apanel: setlcd %x\n",num);
#endif
    setlcd(num);
    return 0;

  case 'r':
  case 'R':
    outb(0x477,inb(0x477)|0x8);		/* Disable first */
    outb(0x477,inb(0x477)&0xf7);	/* Now enable hardware apanel access */
    return 0;
  }
#ifdef DEBUG
  printf("apanel: unknown part '%c'\n",part);
#endif
  return 0;
}


int
mydev_read(dev_t dev, struct uio *uio, int ioflag)
{
  int rv;
  char buf[256];
  unsigned int btn,btn1;

  btn=getbtn(&btn1);
  sprintf(buf,"%4.4x-%4.4x   ",btn,btn1);
#ifdef DEBUG
  printf("apanel: buttonstatus %s\n",buf);
#endif

  uio->uio_rw=UIO_READ;

  if ((rv=uiomove((caddr_t)buf,strlen(buf),uio))!=0) {
    return rv;
  }
  
  return 0;
}
